@@ -8,10 +8,10 @@ use tracing_subscriber::filter::{EnvFilter, LevelFilter};
 
 pub fn start_logger(default_level: LevelFilter) {
     let filter = match EnvFilter::try_from_default_env() {
-        Ok(filter) => filter.add_directive("mio=off".parse().unwrap()),
+        Ok(filter) => filter.add_directive("tokio_util=off".parse().unwrap()),
         _ => EnvFilter::default()
             .add_directive(default_level.into())
-            .add_directive("mio=off".parse().unwrap()),
+            .add_directive("tokio_util=off".parse().unwrap()),
     };
 
     tracing_subscriber::fmt()
