@@ -1,6 +1,6 @@
 use crate::{
     connections::{Connection, ConnectionSide, Connections},
-    protocols::Protocols,
+    protocols::{Protocol, Protocols},
     Config, KnownPeers, Stats,
 };
 
@@ -32,7 +32,7 @@ macro_rules! enable_protocol {
 
             match conn_retriever.await {
                 Ok(Ok(conn)) => conn,
-                Err(_) => unreachable!(), // protocol's task is down! can't recover
+                Err(_) => return Err(io::ErrorKind::BrokenPipe.into()),
                 Ok(e) => return e,
             }
         } else {
@@ -99,11 +99,14 @@ impl Node {
                     Ok(listener) => listener,
                     Err(e) => {
                         if config.allow_random_port {
-                            warn!(parent: span.clone(), "trying any port, the desired one is unavailable: {}", e);
+                            warn!(
+                                parent: &span,
+                                "trying any port, the desired one is unavailable: {}", e
+                            );
                             let random_available_addr = SocketAddr::new(listener_ip, 0);
                             TcpListener::bind(random_available_addr).await?
                         } else {
-                            error!(parent: span.clone(), "the desired port is unavailable: {}", e);
+                            error!(parent: &span, "the desired port is unavailable: {}", e);
                             return Err(e);
                         }
                     }
@@ -143,7 +146,7 @@ impl Node {
         }));
 
         if let Some(listener) = listener {
-            // Use a channel to know when the listening task is ready.
+            // use a channel to know when the listening task is ready
             let (tx, rx) = oneshot::channel::<()>();
 
             let node_clone = node.clone();
@@ -227,11 +230,9 @@ impl Node {
         let mut conn = enable_protocol!(handshake_handler, self, conn);
 
         // split the stream after the handshake
-        if let Some(stream) = conn.stream.take() {
-            let (reader, writer) = stream.into_split();
-            conn.reader = Some(reader);
-            conn.writer = Some(writer);
-        }
+        let (reader, writer) = conn.stream.take().unwrap().into_split();
+        conn.reader = Some(reader);
+        conn.writer = Some(writer);
 
         let conn = enable_protocol!(reading_handler, self, conn);
         let conn = enable_protocol!(writing_handler, self, conn);
@@ -265,14 +266,17 @@ impl Node {
         // enact the enabled protocols
         let mut connection = self.enable_protocols(connection).await?;
 
-        // the protocols are responsible for doing reads and writes; ensure that the Connection object
-        // is not capable of performing them if the protocols haven't been enabled.
-        connection.reader = None;
-        connection.writer = None;
+        // if Reading is enabled, we'll notify the related task when the connection is fully ready
+        let conn_ready_tx = connection.readiness_notifier.take();
 
         self.connections.add(connection);
         self.connecting.lock().remove(&peer_addr);
 
+        // send the aforementioned notification so that reading from the socket can commence
+        if let Some(tx) = conn_ready_tx {
+            let _ = tx.send(());
+        }
+
         Ok(())
     }
 
@@ -341,11 +345,6 @@ impl Node {
                 task.abort();
             }
 
-            // drop the associated outbound message sender if Writing is enabled
-            if let Some(handler) = self.protocols.writing_handler.get() {
-                handler.senders.write().remove(&addr);
-            }
-
             // if the (owning) node was not the initiator of the connection, it doesn't know the listening address
             // of the associated peer, so the related stats are unreliable; the next connection initiated by the
             // peer could be bound to an entirely different port number
