@@ -1,4 +1,7 @@
-use crate::{protocols::ReturnableConnection, Connection, Node, Pea2Pea};
+use crate::{
+    protocols::{ProtocolHandler, ReturnableConnection},
+    Node, Pea2Pea,
+};
 
 #[cfg(doc)]
 use crate::{protocols::Handshake, Config};
@@ -7,14 +10,13 @@ use async_trait::async_trait;
 use bytes::BytesMut;
 use futures_util::StreamExt;
 use tokio::{
-    net::tcp::OwnedReadHalf,
+    io::AsyncRead,
     sync::{mpsc, oneshot},
-    time::sleep,
 };
 use tokio_util::codec::{Decoder, FramedRead};
 use tracing::*;
 
-use std::{io, net::SocketAddr, time::Duration};
+use std::{io, net::SocketAddr};
 
 /// Can be used to specify and enable reading, i.e. receiving inbound messages. If the [`Handshake`]
 /// protocol is enabled too, it goes into force only after the handshake has been concluded.
@@ -35,7 +37,7 @@ where
     async fn enable_reading(&self) {
         let (conn_sender, mut conn_receiver) = mpsc::unbounded_channel::<ReturnableConnection>();
 
-        // Use a channel to know when the reading task is ready.
+        // use a channel to know when the reading task is ready
         let (tx_reading, rx_reading) = oneshot::channel::<()>();
 
         // the main task spawning per-connection tasks reading messages from their streams
@@ -48,10 +50,14 @@ where
             while let Some((mut conn, conn_returner)) = conn_receiver.recv().await {
                 let addr = conn.addr();
                 let codec = self_clone.codec(addr);
-                let reader = self_clone.take_reader(&mut conn);
+                let reader = conn.reader.take().unwrap();
                 let framed = FramedRead::new(reader, codec);
                 let mut framed = self_clone.map_codec(framed, addr);
 
+                // the connection will notify the reading task once it's fully ready
+                let (tx_conn_ready, rx_conn_ready) = oneshot::channel();
+                conn.readiness_notifier = Some(tx_conn_ready);
+
                 let initial_read_buffer_size = self_clone.node().config().initial_read_buffer_size;
                 if initial_read_buffer_size != 0 {
                     framed.read_buffer_mut().reserve(initial_read_buffer_size);
@@ -60,7 +66,7 @@ where
                 let (inbound_message_sender, mut inbound_message_receiver) =
                     mpsc::channel(self_clone.node().config().inbound_queue_depth);
 
-                // Use a channel to know when the processing task is ready.
+                // use a channel to know when the processing task is ready
                 let (tx_processing, rx_processing) = oneshot::channel::<()>();
 
                 // the task for processing parsed messages
@@ -80,21 +86,18 @@ where
                 let _ = rx_processing.await;
                 conn.tasks.push(inbound_processing_task);
 
-                // Use a channel to know when the reader task is ready.
+                // use a channel to know when the reader task is ready
                 let (tx_reader, rx_reader) = oneshot::channel::<()>();
 
                 // the task for reading messages from a stream
-                let reader_clone = self_clone.clone();
+                let node = self_clone.node().clone();
                 let reader_task = tokio::spawn(async move {
-                    let node = reader_clone.node();
                     trace!(parent: node.span(), "spawned a task for reading messages from {}", addr);
                     tx_reader.send(()).unwrap(); // safe; the channel was just opened
 
                     // postpone reads until the connection is fully established; if the process fails,
                     // this task gets aborted, so there is no need for a dedicated timeout
-                    while !node.connected_addrs().contains(&addr) {
-                        sleep(Duration::from_millis(1)).await;
-                    }
+                    let _ = rx_conn_ready.await;
 
                     while let Some(bytes) = framed.next().await {
                         match bytes {
@@ -111,11 +114,6 @@ where
                                 if node.config().fatal_io_errors.contains(&e.kind()) {
                                     node.disconnect(addr).await;
                                     break;
-                                } else {
-                                    sleep(Duration::from_secs(
-                                        node.config().invalid_read_delay_secs,
-                                    ))
-                                    .await;
                                 }
                             }
                         }
@@ -135,8 +133,8 @@ where
         let _ = rx_reading.await;
         self.node().tasks.lock().push(reading_task);
 
-        // register the ReadingHandler with the Node
-        let hdl = ReadingHandler(conn_sender);
+        // register the Reading handler with the Node
+        let hdl = Box::new(ProtocolHandler(conn_sender));
         assert!(
             self.node().protocols.reading_handler.set(hdl).is_ok(),
             "the Reading protocol was enabled more than once!"
@@ -145,11 +143,11 @@ where
 
     /// Wraps the user-supplied [`Decoder`] in another one used for message accounting.
     #[doc(hidden)]
-    fn map_codec(
+    fn map_codec<T: AsyncRead>(
         &self,
-        framed: FramedRead<OwnedReadHalf, Self::Codec>,
+        framed: FramedRead<T, Self::Codec>,
         addr: SocketAddr,
-    ) -> FramedRead<OwnedReadHalf, CountingCodec<Self::Codec>> {
+    ) -> FramedRead<T, CountingCodec<Self::Codec>> {
         framed.map_decoder(|codec| CountingCodec {
             codec,
             node: self.node().clone(),
@@ -163,13 +161,6 @@ where
 
     /// Processes an inbound message. Can be used to update state, send replies etc.
     async fn process_message(&self, source: SocketAddr, message: Self::Message) -> io::Result<()>;
-
-    /// Obtains the read half of the connection to be used in [`Reading::enable_reading`].
-    fn take_reader(&self, conn: &mut Connection) -> OwnedReadHalf {
-        conn.reader
-            .take()
-            .expect("Connection's reader is not available!")
-    }
 }
 
 /// A wrapper [`Decoder`] that also counts the inbound messages.
@@ -208,14 +199,3 @@ impl<D: Decoder> Decoder for CountingCodec<D> {
         Ok(ret)
     }
 }
-
-/// The handler object dedicated to the [`Reading`] protocol.
-pub(crate) struct ReadingHandler(mpsc::UnboundedSender<ReturnableConnection>);
-
-impl ReadingHandler {
-    pub(crate) fn trigger(&self, item: ReturnableConnection) {
-        if self.0.send(item).is_err() {
-            unreachable!(); // protocol's task is down! can't recover
-        }
-    }
-}
