@@ -1,4 +1,7 @@
-use crate::{protocols::ReturnableConnection, Connection, Pea2Pea};
+use crate::{
+    protocols::{Protocol, ProtocolHandler, ReturnableConnection},
+    Connection, Pea2Pea,
+};
 
 #[cfg(doc)]
 use crate::{protocols::Handshake, Config, Node};
@@ -8,13 +11,14 @@ use futures_util::sink::SinkExt;
 use parking_lot::RwLock;
 use tokio::{
     io::AsyncWrite,
-    net::tcp::OwnedWriteHalf,
     sync::{mpsc, oneshot},
 };
 use tokio_util::codec::{Encoder, FramedWrite};
 use tracing::*;
 
-use std::{any::Any, collections::HashMap, io, net::SocketAddr};
+use std::{any::Any, collections::HashMap, io, net::SocketAddr, sync::Arc};
+
+type WritingSenders = Arc<RwLock<HashMap<SocketAddr, mpsc::Sender<WrappedMessage>>>>;
 
 /// Can be used to specify and enable writing, i.e. sending outbound messages. If the [`Handshake`]
 /// protocol is enabled too, it goes into force only after the handshake has been concluded.
@@ -35,11 +39,14 @@ where
     async fn enable_writing(&self) {
         let (conn_sender, mut conn_receiver) = mpsc::unbounded_channel::<ReturnableConnection>();
 
-        // Use a channel to know when the writing task is ready.
+        let conn_senders: WritingSenders = Default::default();
+
+        // use a channel to know when the writing task is ready
         let (tx_writing, rx_writing) = oneshot::channel::<()>();
 
         // the task spawning tasks reading messages from the given stream
         let self_clone = self.clone();
+        let conn_senders_clone = conn_senders.clone();
         let writing_task = tokio::spawn(async move {
             trace!(parent: self_clone.node().span(), "spawned the Writing handler task");
             tx_writing.send(()).unwrap(); // safe; the channel was just opened
@@ -48,22 +55,24 @@ where
             while let Some((mut conn, conn_returner)) = conn_receiver.recv().await {
                 let addr = conn.addr();
                 let codec = self_clone.codec(addr);
-                let writer = self_clone.take_writer(&mut conn);
+                let writer = conn.writer.take().unwrap();
                 let mut framed = FramedWrite::new(writer, codec);
 
                 let (outbound_message_sender, mut outbound_message_receiver) =
                     mpsc::channel(self_clone.node().config().outbound_queue_depth);
 
-                if let Some(handler) = self_clone.node().protocols.writing_handler.get() {
-                    handler
-                        .senders
-                        .write()
-                        .insert(addr, outbound_message_sender);
-                } else {
-                    unreachable!();
-                }
+                // register the connection's message sender with the Writing protocol handler
+                conn_senders_clone
+                    .write()
+                    .insert(addr, outbound_message_sender);
 
-                // Use a channel to know when the writer task is ready.
+                // this will automatically drop the sender upon a disconnect
+                let auto_cleanup = SenderCleanup {
+                    addr,
+                    senders: Arc::clone(&conn_senders_clone),
+                };
+
+                // use a channel to know when the writer task is ready
                 let (tx_writer, rx_writer) = oneshot::channel::<()>();
 
                 // the task for writing outbound messages
@@ -73,6 +82,9 @@ where
                     trace!(parent: node.span(), "spawned a task for writing messages to {}", addr);
                     tx_writer.send(()).unwrap(); // safe; the channel was just opened
 
+                    // move the cleanup into the task that gets aborted on disconnect
+                    let _auto_cleanup = auto_cleanup;
+
                     while let Some(wrapped_msg) = outbound_message_receiver.recv().await {
                         let msg = wrapped_msg.msg.downcast::<Self::Message>().unwrap();
 
@@ -108,10 +120,10 @@ where
         self.node().tasks.lock().push(writing_task);
 
         // register the WritingHandler with the Node
-        let hdl = WritingHandler {
-            handler: conn_sender,
-            senders: Default::default(),
-        };
+        let hdl = Box::new(WritingHandler {
+            handler: ProtocolHandler(conn_sender),
+            senders: conn_senders,
+        });
         assert!(
             self.node().protocols.writing_handler.set(hdl).is_ok(),
             "the Writing protocol was enabled more than once!"
@@ -195,13 +207,6 @@ where
             Err(io::ErrorKind::Unsupported.into())
         }
     }
-
-    /// Obtains the write half of the connection to be used in [`Writing::enable_writing`].
-    fn take_writer(&self, conn: &mut Connection) -> OwnedWriteHalf {
-        conn.writer
-            .take()
-            .expect("Connection's writer is not available!")
-    }
 }
 
 /// Used to queue messages for delivery.
@@ -223,15 +228,24 @@ impl WrappedMessage {
 }
 
 /// The handler object dedicated to the [`Writing`] protocol.
-pub struct WritingHandler {
-    handler: mpsc::UnboundedSender<ReturnableConnection>,
-    pub(crate) senders: RwLock<HashMap<SocketAddr, mpsc::Sender<WrappedMessage>>>,
+pub(crate) struct WritingHandler {
+    handler: ProtocolHandler<Connection, io::Result<Connection>>,
+    pub(crate) senders: WritingSenders,
 }
 
-impl WritingHandler {
-    pub(crate) fn trigger(&self, item: ReturnableConnection) {
-        if self.handler.send(item).is_err() {
-            unreachable!(); // protocol's task is down! can't recover
-        }
+impl Protocol<Connection, io::Result<Connection>> for WritingHandler {
+    fn trigger(&self, item: ReturnableConnection) {
+        self.handler.trigger(item);
+    }
+}
+
+pub(crate) struct SenderCleanup {
+    addr: SocketAddr,
+    senders: WritingSenders,
+}
+
+impl Drop for SenderCleanup {
+    fn drop(&mut self) {
+        self.senders.write().remove(&self.addr);
     }
 }
